import React, { useState, useEffect } from "react";
import { useSelector } from "react-redux";
import DashboardLayout from "../common/DashboardLayout";
import { MobileDisplay, DesktopDisplay } from "../mobile/MediaQuery";
import TableHeader from "../common/TableHeader";
import PlusIcon from "../../../img/alerts/alert-index-plus-icon.svg";
import AllIcon from "../../../img/alerts/alert-index-all-icon.svg";
import AlertIcon from "../../../img/alerts/alert-index-add-icon.svg";
import { minWidth } from "../../util/constants";
import AlertForm from "../alerts/AlertForm";
import AlertIndexTable from "./AlertIndexTable";
import AlertTypeButton from "./AlertTypeButton";
import AlertsBar from "./AlertsBar";
import { ALL_ALERTS } from "../../graphql/alerts";
import { useQuery } from "@apollo/client";
import { SkeletonLayout } from "../common/SkeletonLayout";
import DeleteAlertModal from "./DeleteAlertModal";
import { useHistory } from "react-router-dom";
import analyticsLogger from "../../util/analyticsLogger";
import ErrorMessage from "../common/ErrorMessage";

export default (props) => {
  const history = useHistory();
  const [showPage, setShowPage] = useState("allAlerts");
  const [alertType, setAlertType] = useState(null);
  const [showDeleteAlertModal, setShowDeleteAlertModal] = useState(false);
  const [selectedAlert, setSelectedAlert] = useState(null);
  const { loading, data, error, refetch } = useQuery(ALL_ALERTS, {
    fetchPolicy: "cache-and-network",
  });
  const alertsData = data ? data.allAlerts : [];

  const socket = useSelector((state) => state.apollo.socket);
  const alertsChannel = socket.channel("graphql:alerts_index_table", {});
  const currentOrganizationId = useSelector(
    (state) => state.organization.currentOrganizationId
  );

  useEffect(() => {
    // executed when mounted
    alertsChannel.join();
    alertsChannel.on(
      `graphql:alerts_index_table:${currentOrganizationId}:alert_list_update`,
      (_message) => {
        refetch();
      }
    );
    analyticsLogger.logEvent("ACTION_NAV_ALERTS_INDEX");

    // executed when unmounted
    return () => {
      alertsChannel.leave();
    };
  }, []);

  useEffect(() => {
    if (props.match.params.id) {
      setShowPage("showAlert");
    } else if (props.match.path === "/alerts/new") {
      setShowPage("new");
    }
  });

  const openDeleteAlertModal = (alert) => {
    setShowDeleteAlertModal(true);
    setSelectedAlert(alert);
  };

  const closeDeleteAlertModal = () => {
    setShowDeleteAlertModal(false);
    setSelectedAlert(null);
  };

  return (
    <>
      <MobileDisplay />
      <DesktopDisplay>
        <DashboardLayout title="My Alerts" user={props.user}>
          <TableHeader
            backgroundColor="#D3E0EE"
            otherColor="#ACC6DD"
            homeIcon={null}
            goToAll={() => {
              setShowPage("allAlerts");
              history.push("/alerts");
            }}
            allIcon={AllIcon}
            textColor="#3C6B95"
            allText="All Alerts"
            onHomePage={showPage === "home"}
            onAllPage={showPage === "allAlerts"}
            onNewPage={showPage === "new"}
            addIcon={PlusIcon}
            goToNew={() => {
              setShowPage("new");
              setAlertType(null);
              history.push("/alerts/new");
            }}
            noHome
            borderRadius="25px"
            extraContent={
              <AlertsBar
                shownAlertId={props.match.params.id}
                alerts={alertsData}
              />
            }
            newText="Add New Alert"
          >
            {props.match.params.id && showPage === "showAlert" && (
              <div className="no-scroll-bar" style={{ overflowX: "scroll" }}>
                <div style={{ minWidth }}>
                  <AlertForm
                    key={props.match.params.id}
                    show
                    id={props.match.params.id}
                    back={() => {
                      history.push("/alerts");
                    }}
                  />
                </div>
              </div>
            )}
            {showPage === "new" && alertType === null && (
              <div
                className="blankstateWrapper no-scroll-bar"
                style={{
                  height: "600px",
                  paddingTop: "100px",
                  overflowX: "scroll",
                }}
              >
                <div
                  style={{
                    width: "100%",
                    maxWidth: 500,
                    minWidth: 500,
                    margin: "0 auto",
                    textAlign: "center",
                  }}
                >
                  <img src={AlertIcon} />
                  <h1>Choose Type of Alert</h1>
                  <div
                    style={{
                      padding: "10px 60px 1px 60px",
                      margin: "10px 0px",
                    }}
                  >
                    <p style={{ fontSize: "16px", color: "#565656" }}>
                      Alerts can be created for
                      <br />
                      Device/Label Nodes or Integration Nodes.
                    </p>
                  </div>
                  <div
                    style={{
                      flexDirection: "row",
                      display: "flex",
                      justifyContent: "center",
                    }}
                  >
                    <AlertTypeButton
                      backgroundColor="#2C79EE"
                      onClick={() => {
                        setAlertType("device/label");
                      }}
                      margin={12}
                    >
                      Device/Label Alert
                    </AlertTypeButton>
                    <AlertTypeButton
                      backgroundColor="#12CB9E"
                      onClick={() => {
                        setAlertType("integration");
                      }}
                    >
                      Integration Alert
                    </AlertTypeButton>
                  </div>
                </div>
                <style jsx>{`
                  h1 {
                    font-size: 30px;
                    margin: 10px 0px;
                    font-weight: 600;
                  }
                  p {
                    font-weight: 300;
                    margin-bottom: 10px;
                  }
                `}</style>
              </div>
            )}
            {showPage === "new" && alertType && (
              <div className="no-scroll-bar" style={{ overflowX: "scroll" }}>
                <div style={{ minWidth }}>
                  <AlertForm
                    alertType={alertType}
                    back={() => {
                      setAlertType(null);
                    }}
                  />
                </div>
              </div>
            )}
            {showPage === "allAlerts" && error && <ErrorMessage />}
            {showPage === "allAlerts" && loading && (
              <div style={{ padding: 40 }}>
                <SkeletonLayout />
              </div>
            )}
            {showPage === "allAlerts" && !loading && !error && (
              <AlertIndexTable
                data={alertsData}
                openDeleteAlertModal={openDeleteAlertModal}
                history={history}
              />
            )}
          </TableHeader>
          <DeleteAlertModal
            open={showDeleteAlertModal}
            alert={selectedAlert}
            close={closeDeleteAlertModal}
          />
        </DashboardLayout>
      </DesktopDisplay>
    </>
  );
};
